@@ -1,4 +1,5 @@
-{-# LANGUAGE CPP #-}
+{-# LANGUAGE DeriveGeneric #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  Distribution.Client.Init.Types
@@ -17,6 +18,8 @@ module Distribution.Client.Init.Types where
 import Distribution.Simple.Setup
   ( Flag(..) )
 
+import Distribution.Types.Dependency as P
+import Distribution.Compat.Semigroup
 import Distribution.Version
 import Distribution.Verbosity
 import qualified Distribution.Package as P
@@ -28,9 +31,7 @@ import qualified Text.PrettyPrint as Disp
 import qualified Distribution.Compat.ReadP as Parse
 import Distribution.Text
 
-#if !MIN_VERSION_base(4,8,0)
-import Data.Monoid
-#endif
+import GHC.Generics ( Generic )
 
 -- | InitFlags is really just a simple type to represent certain
 --   portions of a .cabal file.  Rather than have a flag for EVERY
@@ -71,77 +72,27 @@ data InitFlags =
               , initVerbosity :: Flag Verbosity
               , overwrite     :: Flag Bool
               }
-  deriving (Show)
+  deriving (Show, Generic)
 
   -- the Monoid instance for Flag has later values override earlier
   -- ones, which is why we want Maybe [foo] for collecting foo values,
   -- not Flag [foo].
 
-data PackageType = Library | Executable
+data BuildType = LibBuild | ExecBuild
+
+data PackageType = Library | Executable | LibraryAndExecutable
   deriving (Show, Read, Eq)
 
-instance Text PackageType where
-  disp = Disp.text . show
-  parse = Parse.choice $ map (fmap read . Parse.string . show) [Library, Executable]
+displayPackageType :: PackageType -> String
+displayPackageType LibraryAndExecutable = "Library and Executable"
+displayPackageType pkgtype              = show pkgtype
 
 instance Monoid InitFlags where
-  mempty = InitFlags
-    { nonInteractive = mempty
-    , quiet          = mempty
-    , packageDir     = mempty
-    , noComments     = mempty
-    , minimal        = mempty
-    , packageName    = mempty
-    , version        = mempty
-    , cabalVersion   = mempty
-    , license        = mempty
-    , author         = mempty
-    , email          = mempty
-    , homepage       = mempty
-    , synopsis       = mempty
-    , category       = mempty
-    , extraSrc       = mempty
-    , packageType    = mempty
-    , mainIs         = mempty
-    , language       = mempty
-    , exposedModules = mempty
-    , otherModules   = mempty
-    , otherExts      = mempty
-    , dependencies   = mempty
-    , sourceDirs     = mempty
-    , buildTools     = mempty
-    , initVerbosity  = mempty
-    , overwrite      = mempty
-    }
-  mappend  a b = InitFlags
-    { nonInteractive = combine nonInteractive
-    , quiet          = combine quiet
-    , packageDir     = combine packageDir
-    , noComments     = combine noComments
-    , minimal        = combine minimal
-    , packageName    = combine packageName
-    , version        = combine version
-    , cabalVersion   = combine cabalVersion
-    , license        = combine license
-    , author         = combine author
-    , email          = combine email
-    , homepage       = combine homepage
-    , synopsis       = combine synopsis
-    , category       = combine category
-    , extraSrc       = combine extraSrc
-    , packageType    = combine packageType
-    , mainIs         = combine mainIs
-    , language       = combine language
-    , exposedModules = combine exposedModules
-    , otherModules   = combine otherModules
-    , otherExts      = combine otherExts
-    , dependencies   = combine dependencies
-    , sourceDirs     = combine sourceDirs
-    , buildTools     = combine buildTools
-    , initVerbosity  = combine initVerbosity
-    , overwrite      = combine overwrite
-    }
-    where combine field = field a `mappend` field b
+  mempty = gmempty
+  mappend = (<>)
+
+instance Semigroup InitFlags where
+  (<>) = gmappend
 
 -- | Some common package categories.
 data Category
@@ -166,5 +117,5 @@ data Category
 
 instance Text Category where
   disp  = Disp.text . show
-  parse = Parse.choice $ map (fmap read . Parse.string . show) [Codec .. ]
+  parse = Parse.choice $ map (fmap read . Parse.string . show) [Codec .. ] -- TODO: eradicateNoParse
 
