@@ -16,10 +16,12 @@ module Distribution.Client.InstallSymlink (
     symlinkBinary,
   ) where
 
-#if mingw32_HOST_OS
+#ifdef mingw32_HOST_OS
 
 import Distribution.Package (PackageIdentifier)
+import Distribution.Types.UnqualComponentName
 import Distribution.Client.InstallPlan (InstallPlan)
+import Distribution.Client.Types (BuildOutcomes)
 import Distribution.Client.Setup (InstallFlags)
 import Distribution.Simple.Setup (ConfigFlags)
 import Distribution.Simple.Compiler
@@ -28,33 +30,36 @@ import Distribution.System
 symlinkBinaries :: Platform -> Compiler
                 -> ConfigFlags
                 -> InstallFlags
-                -> InstallPlan 
-                -> IO [(PackageIdentifier, String, FilePath)]
-symlinkBinaries _ _ _ _ _ = return []
+                -> InstallPlan
+                -> BuildOutcomes
+                -> IO [(PackageIdentifier, UnqualComponentName, FilePath)]
+symlinkBinaries _ _ _ _ _ _ = return []
 
-symlinkBinary :: FilePath -> FilePath -> String -> String -> IO Bool
+symlinkBinary :: FilePath -> FilePath -> UnqualComponentName -> String -> IO Bool
 symlinkBinary _ _ _ _ = fail "Symlinking feature not available on Windows"
 
 #else
 
 import Distribution.Client.Types
-         ( SourcePackage(..)
-         , GenericReadyPackage(..), ReadyPackage, enableStanzas
-         , ConfiguredPackage(..) , fakeUnitId)
+         ( ConfiguredPackage(..), BuildOutcomes )
 import Distribution.Client.Setup
          ( InstallFlags(installSymlinkBinDir) )
 import qualified Distribution.Client.InstallPlan as InstallPlan
 import Distribution.Client.InstallPlan (InstallPlan)
 
+import Distribution.Solver.Types.SourcePackage
+import Distribution.Solver.Types.OptionalStanza
+
 import Distribution.Package
-         ( PackageIdentifier, Package(packageId), UnitId(..) )
+         ( PackageIdentifier, Package(packageId), UnitId, installedUnitId )
+import Distribution.Types.UnqualComponentName
 import Distribution.Compiler
          ( CompilerId(..) )
 import qualified Distribution.PackageDescription as PackageDescription
 import Distribution.PackageDescription
          ( PackageDescription )
 import Distribution.PackageDescription.Configuration
-         ( finalizePackageDescription )
+         ( finalizePD )
 import Distribution.Simple.Setup
          ( ConfigFlags(..), fromFlag, fromFlagOrDefault, flagToMaybe )
 import qualified Distribution.Simple.InstallDirs as InstallDirs
@@ -62,6 +67,8 @@ import Distribution.Simple.Compiler
          ( Compiler, compilerInfo, CompilerInfo(..) )
 import Distribution.System
          ( Platform )
+import Distribution.Text
+         ( display )
 
 import System.Posix.Files
          ( getSymbolicLinkStatus, isSymbolicLink, createSymbolicLink
@@ -104,8 +111,9 @@ symlinkBinaries :: Platform -> Compiler
                 -> ConfigFlags
                 -> InstallFlags
                 -> InstallPlan
-                -> IO [(PackageIdentifier, String, FilePath)]
-symlinkBinaries platform comp configFlags installFlags plan =
+                -> BuildOutcomes
+                -> IO [(PackageIdentifier, UnqualComponentName, FilePath)]
+symlinkBinaries platform comp configFlags installFlags plan buildOutcomes =
   case flagToMaybe (installSymlinkBinDir installFlags) of
     Nothing            -> return []
     Just symlinkBinDir
@@ -123,31 +131,32 @@ symlinkBinaries platform comp configFlags installFlags plan =
                then return Nothing
                else return (Just (pkgid, publicExeName,
                                   privateBinDir </> privateExeName))
-        | (ReadyPackage (ConfiguredPackage _ _flags _ _) _, pkg, exe) <- exes
+        | (rpkg, pkg, exe) <- exes
         , let pkgid  = packageId pkg
               -- This is a bit dodgy; probably won't work for Backpack packages
-              ipid = fakeUnitId pkgid
+              ipid = installedUnitId rpkg
               publicExeName  = PackageDescription.exeName exe
-              privateExeName = prefix ++ publicExeName ++ suffix
+              privateExeName = prefix ++ unUnqualComponentName publicExeName ++ suffix
               prefix = substTemplate pkgid ipid prefixTemplate
               suffix = substTemplate pkgid ipid suffixTemplate ]
   where
     exes =
       [ (cpkg, pkg, exe)
-      | InstallPlan.Installed cpkg _ _ <- InstallPlan.toList plan
-      , let pkg   = pkgDescription cpkg
+      | InstallPlan.Configured cpkg <- InstallPlan.toList plan
+      , case InstallPlan.lookupBuildOutcome cpkg buildOutcomes of
+          Just (Right _success) -> True
+          _                     -> False
+      , let pkg :: PackageDescription
+            pkg = pkgDescription cpkg
       , exe <- PackageDescription.executables pkg
       , PackageDescription.buildable (PackageDescription.buildInfo exe) ]
 
-    pkgDescription :: ReadyPackage -> PackageDescription
-    pkgDescription (ReadyPackage (ConfiguredPackage
-                                    (SourcePackage _ pkg _ _)
-                                    flags stanzas _)
-                                  _) =
-      case finalizePackageDescription flags
+    pkgDescription (ConfiguredPackage _ (SourcePackage _ pkg _ _)
+                                      flags stanzas _) =
+      case finalizePD flags (enableStanzas stanzas)
              (const True)
-             platform cinfo [] (enableStanzas stanzas pkg) of
-        Left _ -> error "finalizePackageDescription ReadyPackage failed"
+             platform cinfo [] pkg of
+        Left _ -> error "finalizePD ReadyPackage failed"
         Right (desc, _) -> desc
 
     -- This is sadly rather complicated. We're kind of re-doing part of the
@@ -177,30 +186,32 @@ symlinkBinaries platform comp configFlags installFlags plan =
     cinfo            = compilerInfo comp
     (CompilerId compilerFlavor _) = compilerInfoId cinfo
 
-symlinkBinary :: FilePath -- ^ The canonical path of the public bin dir
-                          --   eg @/home/user/bin@
-              -> FilePath -- ^ The canonical path of the private bin dir
-                          --   eg @/home/user/.cabal/bin@
-              -> String   -- ^ The name of the executable to go in the public
-                          --   bin dir, eg @foo@
-              -> String   -- ^ The name of the executable to in the private bin
-                          --   dir, eg @foo-1.0@
-              -> IO Bool  -- ^ If creating the symlink was successful. @False@
-                          --   if there was another file there already that we
-                          --   did not own. Other errors like permission errors
-                          --   just propagate as exceptions.
+symlinkBinary ::
+  FilePath               -- ^ The canonical path of the public bin dir eg
+                         --   @/home/user/bin@
+  -> FilePath            -- ^ The canonical path of the private bin dir eg
+                         --   @/home/user/.cabal/bin@
+  -> UnqualComponentName -- ^ The name of the executable to go in the public bin
+                         --   dir, eg @foo@
+  -> String              -- ^ The name of the executable to in the private bin
+                         --   dir, eg @foo-1.0@
+  -> IO Bool             -- ^ If creating the symlink was successful. @False@ if
+                         --   there was another file there already that we did
+                         --   not own. Other errors like permission errors just
+                         --   propagate as exceptions.
 symlinkBinary publicBindir privateBindir publicName privateName = do
-  ok <- targetOkToOverwrite (publicBindir </> publicName)
+  ok <- targetOkToOverwrite (publicBindir </> publicName')
                             (privateBindir </> privateName)
   case ok of
     NotOurFile    ->                     return False
     NotExists     ->           mkLink >> return True
     OkToOverwrite -> rmLink >> mkLink >> return True
   where
+    publicName' = display publicName
     relativeBindir = makeRelative publicBindir privateBindir
     mkLink = createSymbolicLink (relativeBindir </> privateName)
-                                (publicBindir   </> publicName)
-    rmLink = removeLink (publicBindir </> publicName)
+                                (publicBindir   </> publicName')
+    rmLink = removeLink (publicBindir </> publicName')
 
 -- | Check a file path of a symlink that we would like to create to see if it
 -- is OK. For it to be OK to overwrite it must either not already exist yet or
